function Depth = estimate_depth(I1,I2,Hp,visualize)
% function Depth = estimate_depth(I1,I2,Hp,visualize)
%
% This function returns a depth estimate from
% images I1 (near focused) and I2 (far focused). 
% Depth estimation is performed by extracting
% the orthogonal operators family from HHT, which can
% be passed as an argument, or will be retrieved from
% the file Operators.mat
%
% Copyright 2006 Paolo Favaro (p.favaro@hw.ac.uk)
% 
% School of Engineering and Physical Sciences
% Heriot-Watt University, Edinburgh, UK
% 
% Last revision: May 2006
%
% This program can be used only for research purposes.
% This program is distributed WITHOUT ANY WARRANTY; 
% without even the implied warranty of MERCHANTABILITY 
% or FITNESS FOR A PARTICULAR PURPOSE.

On = 1;
Off = 0;
if nargin<4
    % visualize partial results while estimating depth map (slow)
    visualize = Off;
elseif (visualize ~= On) & (visualize ~= Off)
    % visualize partial results while estimating depth map (slow)
    visualize = Off;    
end
if nargin<3
   fid=fopen('Operators.mat','r');
   fclose(fid);
   if fid==-1
      error('No operators found. Run generate_operators first.')
   end
   load Operators
end
if nargin<2
   error('Too few arguments!');
end
numlevels = size(Hp,3);
ni = sqrt(size(Hp,1)/2);
if ni~=floor(ni)
   error('Orthogonal operators have incorrect size!');
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% image linear prefiltering %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% eliminate brightness component
dI1 = del2(I1);
dI2 = del2(I2);
[mI,nI] = size(I1);
% compute depth level
fprintf('Computing depth levels\n');
fprintf('%i Iterations',numlevels)
Cost = zeros(mI*nI,numlevels);
C = zeros(mI*nI,size(Hp,1));
for i = 1:numlevels
   for k=1:size(Hp,1)
      P1 = reshape(Hp(k,ni*ni:-1:1,i,1),ni,ni);
      P2 = reshape(Hp(k,ni*ni+[ni*ni:-1:1],i,1),ni,ni);
      C(:,k) = reshape(conv2(dI1,P1,'same')+...
          conv2(dI2,P2,'same'),mI*nI,1);
   end
   Flt = C*Hp(:,:,i);
   Cost(:,i) = sum(Flt.^2,2);
   if visualize
      [dummy,Depth] = min(Cost(:,1:i),[],2);
      Depth = reshape(Depth,mI,nI);
      % visualize result
      figure(1)
      set(gcf,'name',sprintf('Iteration %3i out of %3i',i,numlevels));
      subplot(221)
      image(I1)
      axis equal
      axis off
      set(gca,'XTick',[],'YTick',[]);
      set(gca,'Box','off');
      colormap gray(256)
      title('Input #1')
      drawnow;
      subplot(222)
      image(I2)
      axis equal
      axis off
      set(gca,'XTick',[],'YTick',[]);
      set(gca,'Box','off');
      colormap gray(256)
      title('Input #2')
      drawnow;
      step = ceil(min([size(Depth,1) ...
             size(Depth,2)])/50);
      pix2mm = 1/2;
      [X,Y] = meshgrid(pix2mm*[1:step:size(Depth,2)],...
              pix2mm*[1:step:size(Depth,1)]);
      axisZ = [0 numlevels];
      subplot(223)
      h = surf(X,-Depth(1:step:end,1:step:end),-Y,Depth(1:step:end,1:step:end));
      shading interp
      hold off
      axis([0 nI -axisZ(2) -axisZ(1) -mI 0])
      set(gca,'XTick',[],'YTick',[],'ZTick',[]);
      colormap gray(256)
      title('Estimated Depth Map')
      view([-1 -2 1]);
      axis equal
      axis off
      set(h,'EdgeColor','r');
      drawnow;
      subplot(224)
      imagesc(Depth);
      colormap gray(256)
      %axis off
      axis equal
      set(gca,'XTick',[],'YTick',[]);
      set(gca,'Box','off');
      title('Estimated Depth Map')
      drawnow;
   end
end
% find minimum of the cost functional
[dummy,Depth] = min(Cost,[],2);
Depth = reshape(Depth,mI,nI);
fprintf('\ndone.\n');
return